//
// file			: adc.c
// author	    : Ian Forse
// date			: 12-APR-2004
// purpose	: ADC setup, reading and format conversion
// includes 	: P16F877.h
// notes     : 
// compiler  : FED PIC C Compiler 9.10
//
// Copyright (C) 2004 Ian Forse
//
// This library is free software; you can redistribute it and/or
// modify it under the terms of the GNU Lesser General Public
// License as published by the Free Software Foundation; either
// version 2.1 of the License, or (at your option) any later version.
//
// This library is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
// Lesser General Public License for more details.

// You should have received a copy of the GNU Lesser General Public
// License along with this library; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
// USA


#include <P16F877.h>	// Processor header

// ----------------------------------------------------------------------------
// Converts binary (0x0000 to 0x03ff) to Binary coded decimal (0000 to 1023).
// Value returned as a single 16bit word.
// ----------------------------------------------------------------------------

unsigned int bintobcd (unsigned int p)
{                                          
	unsigned int w, r, i;
	i=16;   
	r=0; 
		while(1)
		{
			r<<=1;
			if (p & 0x8000) r|=1;
			p<<=1;
			if (--i ==0 ) break;
			w=0x03+r; 
			if (w & 0x08) r=w;
			w=0x30+r;
			if (w & 0x80) r=w;
			w=0x0300u+r;
			if (w & 0x0800u) r=w;	
			w=0x3000u+r;
			if (w & 0x8000u) r=w;	
		}
			return r;
}

// -------------------------------------------------------------------
// Initialising the ADC channels 
// Select the setting that you require as per 16F877 datasheet, DS30292
// --------------------------------------------------------------------

void adc_init (void)
{
ADCON1 = 0x80;	// SELECT ONLY ONE FROM BELOW.
}

// procedures for select combinations of ad inputs ( DS30292 page 122 )
// without external reference, right justified
         
// ADCON1 = 0x8E;	// a0 adc input,a1,a2,a3,a5,e0,e1,e2 digital I/O
 
// ADCON1 = 0x84;	// a0,a1,a3 adc inputs,a2,a5,e0,e1,e2 digital I/O

// ADCON1 = 0x82;	// a0,a1,a2,a3,a4 adc inputs, a5,a6,a7 digital I/O

// ADCON1 = 0x89;	// a0,a1,a2,a3,a4,a5 adc inputs, a6,a7 digital I/O

// ADCON1 = 0x80;	// all adc inputs

 
// select combinations of ad inputs with positive reference on a3,
// right justified ( DS30292A page 121 )

// ADCON1 = 0x85;	// a0,a1 adc inputs,a4,a5,a6,a7 digital I/O

// ADCON1 = 0x83;	// a0,a1,a2,a4 adc inputs, a5,a6,a7 digital I/O

// ADCON1 = 0x8A;	// a0,a1,a2,a4,a5 adc inputs, a6,a7 digital I/O

// ADCON1 = 0x81;	// all adc inputs


// select combinations of ad inputs with negative reference on a2 and positive 
// reference on a3, right justified ( DS30292A page 121 ) 


// ADCON1 = 0x8F;	// a0 adc input, a1,a4,a5,a6,a7 digital I/O

// ADCON1 = 0x8D;	// a0,a1 adc input, a4,a5,a6,a7 digital I/O
 
// ADCON1 = 0x8C;	// a0,a1,a4 adc input, a5,a6,a7 digital I/O

// ADCON1 = 0x8A;	// a0,a1,a4,a5 adc input, a6,a7 digital I/O
 
// ADCON1 = 0x88;	// a0,a1,a4,a5,a6,a7 adc input


// make all port_a digital IO


// ADCON1 = 0x87 -- all digital IO


// ----------------------------------------------------------------------------
// Reads the ADC level input on a specified ADC channel 64 times.
// Approximate time for one sample = 581uS
// Averages value to remove noise
// Calls conversion to obtain BCD
// Returns single 16bit word
// ----------------------------------------------------------------------------

unsigned int ReadADC (unsigned char Channel)
{
	float ADC_FILT;	
	unsigned int ADC_VALUE;
	unsigned char i;
	
		if	(Channel == 0) ADCON0 = 0x41;
		else if	(Channel == 1) ADCON0 = 0x49;
		else if	(Channel == 2) ADCON0 = 0x51;
		else if	(Channel == 3) ADCON0 = 0x59;
		else if	(Channel == 4) ADCON0 = 0x61;
		else if	(Channel == 5) ADCON0 = 0x69;
		else if	(Channel == 6) ADCON0 = 0x71;
		else if	(Channel == 6) ADCON0 = 0x79;

// channel 0 = Adj Pot on dev board
// channel 1 = temperature sensor on dev board
// channel 2 = 

		ADC_FILT = 0;			// Clear out previous count
		for (i=0; i<64; i++)
		{
			PIE1 &=~(1<<ADIE);	// Masking the interrupt
			PIR1 &=~(1<<ADIF);	// Resetting the ADC interupt bit					
			ADRESL	= 0; 		// Resetting the ADRES value register
			ADRESH	= 0; 						 
			ADCON0|=(1<<GO);  	// Staring the ADC process
			while (!(PIR1 & 0x40)) continue;   // Wait for conversion complete
			ADC_VALUE = ADRESH; 	// ADC registers are split 2 and 8 bit
			ADC_VALUE = ADC_VALUE << 8;				// Combined into a single integer
			ADC_VALUE += ADRESL;
			ADC_FILT += ADC_VALUE;		 	
			}
			ADC_FILT = (ADC_FILT/64)+0.5;	// Average of 64 readings to reduce noise
							// +0.5 is for rounding			
//			ALTERNATIVE FILTER.  Change i to <255 in the for statement above
//			Comment out 3 lines below and replace with 2 lines below.  
//			Adjust delay in analog1.c
//
//			Software low pass filter to attenuate noise by -12dB
//			ADC_FILT = ADC_FILT + ((ADC_VALUE - ADC_FILT) * 0.0625);
//			}

			 ADC_VALUE = (unsigned int) (ADC_FILT*4.83);// Scaled to read 0 - 5V
			 ADC_VALUE = bintobcd (ADC_VALUE);	// convert to BCD
			 return ADC_VALUE;		// Return the value of the ADC process 
}

