//
// file      : mdm.c
// author    : Ian Forse
// date      : 12-APR-2004
// purpose   : MDM-1 display driver
// includes  : None
// notes     : 
// compiler  : FED PIC C Compiler 9.10
//
// Copyright (C) 2004 Ian Forse
//
// This library is free software; you can redistribute it and/or
// modify it under the terms of the GNU Lesser General Public
// License as published by the Free Software Foundation; either
// version 2.1 of the License, or (at your option) any later version.
//
// This library is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
// Lesser General Public License for more details.

// You should have received a copy of the GNU Lesser General Public
// License along with this library; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
// USA

// ----------------------------------------------------------------------------
// Channel A segment lookup table
// ----------------------------------------------------------------------------

const unsigned char tablea [] = {
0x00,0xE2,0x1E,0x42,0x1C,0x4A,0x16,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x08,0x40,0x32,0x7A,0x62,0x2E,0x00,0x00,0x00,0x02,0x02,0x00,0x02,0x01,0x00,
0xFC,0x90,0xCE,0xD8,0xB2,0x7A,0x7E,0xD0,0xFE,0xFA,0x00,0x01,0x00,0x0A,0x00,0xE1,
0xEC,0xF6,0xD8,0x6C,0xD8,0x6E,0x66,0x7C,0xB6,0x48,0x9C,0x26,0x2C,0xB4,0xB4,0xFC,
0xE6,0xFC,0xE6,0x7A,0x40,0xBC,0x24,0xBC,0x00,0x00,0x48,0x6C,0x00,0xD8,0x00,0x08,
0x00,0xF6,0x3E,0x0E,0x9E,0xEE,0x66,0xFA,0x26,0x0A,0x98,0x26,0x24,0x16,0x06,0x1E,
0xE6,0xF2,0x00,0x08,0x2E,0x1C,0x04,0x14,0x00,0x90,0x0A,0x02,0x00,0x00,0x30,0xFF};


const unsigned char tableb[] = {
0xE2,0x1E,0x42,0x1C,0x4A,0x16,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x48,0x40,0x4A,0x4A,0xCE,0x48,0x40,0x90,0x24,0xFE,0x4A,0x00,0x02,0x00,0x84,
0x00,0x00,0x02,0x02,0x02,0x02,0x02,0x00,0x02,0x02,0x01,0x01,0x90,0x02,0x24,0x0A,
0x42,0x02,0x4A,0x00,0x48,0x00,0x00,0x02,0x02,0x48,0x00,0x90,0x00,0xA0,0x30,0x00,
0x02,0x10,0x12,0x02,0x48,0x00,0x84,0x08,0xB4,0xA8,0x84,0x00,0x30,0x00,0xA0,0x00,
0x40,0x02,0x02,0x02,0x02,0x02,0x00,0x02,0x08,0x0A,0x00,0x12,0x00,0x0A,0x08,0x02,
0x02,0x02,0x0A,0x12,0x00,0x00,0x04,0x14,0xB4,0x42,0x04,0x90,0x48,0x26,0x30,0xFF};


// digit segments values
unsigned char mdm_segments_1a, mdm_segments_1b;
unsigned char mdm_segments_2a, mdm_segments_2b;
unsigned char mdm_segments_3a, mdm_segments_3b;
unsigned char mdm_segments_4a, mdm_segments_4b;
unsigned char mdm_segments_5a, mdm_segments_5b;

// current position for mdm_write()
unsigned char mdm_cursor_position;

// ----------------------------------------------------------------------------
// Shift byte x out to the display as serial.  n is number of bits (5 x 8) + 
// (1 * 3).  Note range is from 0 i.e. 8 is from 0 to 7.
// ----------------------------------------------------------------------------

void mdm_output_byte( unsigned char n, unsigned char x )
{
	signed char i;
		for(i=n; i>=0; i--)
		{
			if ((x>>i)&0x01)
			{
				bRB4 = 1;
			}
			else
			{
				bRB4 = 0;
			}
			bRB5 = 1;
			bRB5 = 0;
		}
}

// ----------------------------------------------------------------------------
// Update the display from the segment values.
// ----------------------------------------------------------------------------

void mdm_update_from_segments(void)
{
	mdm_output_byte( 7, mdm_segments_5a );
	mdm_output_byte( 7, mdm_segments_4a );
	mdm_output_byte( 7, mdm_segments_3a );
	mdm_output_byte( 7, mdm_segments_2a );
	mdm_output_byte( 7, mdm_segments_1a );
	mdm_output_byte( 2, 0x00 );
	bRB2 = 1;
	bRB2 = 0; 
    
	mdm_output_byte( 7, mdm_segments_5b );
	mdm_output_byte( 7, mdm_segments_4b );
	mdm_output_byte( 7, mdm_segments_3b );
	mdm_output_byte( 7, mdm_segments_2b );
	mdm_output_byte( 7, mdm_segments_1b );
	mdm_output_byte( 2, 0x01 );
	bRB2 = 1;
	bRB2 = 0; 
}


// ----------------------------------------------------------------------------
// Adjust segment values to either all on or all off.  Could be dropped if code
// space was tight.
// ----------------------------------------------------------------------------

void mdm_all( unsigned char x)
{
	unsigned char v;
	if (x)
	{
		v = 0xFF;
	}
	else
	{
		v = 0x00;
	}
	mdm_segments_1a = v;
	mdm_segments_1b = v;
	mdm_segments_2a = v;
	mdm_segments_2b = v;
	mdm_segments_3a = v;
	mdm_segments_3b = v;
	mdm_segments_4a = v;
	mdm_segments_4b = v;
	mdm_segments_5a = v;
	mdm_segments_5b = v;
	mdm_update_from_segments();
}   


// ----------------------------------------------------------------------------
// write ascii character c to position p.
// ----------------------------------------------------------------------------
 
void mdm_write_p ( unsigned char p, unsigned char c )
{
	unsigned char segments_a, segments_b, a, b;
	if (p == 1)
	{
		segments_a = mdm_segments_1a;
		segments_b = mdm_segments_1b;
	}  
	else if (p == 2)
	{
		segments_a = mdm_segments_2a;
		segments_b = mdm_segments_2b;
	}   
	else if (p == 3)
	{
		segments_a = mdm_segments_3a;
		segments_b = mdm_segments_3b;
	}   
	else if (p == 4)
	{
		segments_a = mdm_segments_4a;
		segments_b = mdm_segments_4b;
	}   
	else if (p == 5)
	{ 
		segments_a = mdm_segments_5a;
		segments_b = mdm_segments_5b;
	}

	if (c == ',')			// look for ',' == clear dots
	{
		segments_a = segments_a & ! 0x01;
		segments_b = segments_b & ! 0x01;
	}  
	else
	{        
		c = c & 0x7F;	// tables are 128 entries only, so clear highest bit
		a =  tablea[c];		// lookup channel a  
		b =  tableb[c];		// lookup channel b
	}  
      
	if (( ( a | b ) & 0x01 ) == 0)	// don't shift when writing the dots
	{	
		mdm_cursor_position = mdm_cursor_position + 1;
	}

	segments_a = ( segments_a & 0x01 ) | a;	// don't overwrite the dots
	segments_b = ( segments_b & 0x01 ) | b;

	if (p == 1)
	{ 
		mdm_segments_1a = segments_a;
		mdm_segments_1b = segments_b;
	}  
	else if (p == 2)
	{ 
		mdm_segments_2a = segments_a;
		mdm_segments_2b = segments_b;
	}   
	else if (p == 3)
	{
		mdm_segments_3a = segments_a;
		mdm_segments_3b = segments_b;
	}   
	else if (p == 4)
	{ 
		mdm_segments_4a = segments_a;
		mdm_segments_4b = segments_b;
	}   
	else if (p == 5)
	{ 
		mdm_segments_5a = segments_a;
		mdm_segments_5b = segments_b;
	}

	mdm_update_from_segments();
}


// ----------------------------------------------------------------------------
// clear the display, set cursor position to 1
// ----------------------------------------------------------------------------

void mdm_clear(void)
{
	mdm_all( 0 );   
	mdm_cursor_position = 1;
}


// ----------------------------------------------------------------------------
// write acii character c to cursor position, reset by mdm_clear.  Allows TTY
// style message to move scroll
// ----------------------------------------------------------------------------

void mdm_write(  unsigned char c )
{        
	if (mdm_cursor_position > 5)
	{
		mdm_segments_1a = mdm_segments_2a;
		mdm_segments_1b = mdm_segments_2b;
		mdm_segments_2a = mdm_segments_3a;
		mdm_segments_2b = mdm_segments_3b;
		mdm_segments_3a = mdm_segments_4a;
		mdm_segments_3b = mdm_segments_4b;
		mdm_segments_4a = mdm_segments_5a;
		mdm_segments_4b = mdm_segments_5b;
		mdm_cursor_position = 5;
	}
	mdm_write_p( mdm_cursor_position, c );
}


// ----------------------------------------------------------------------------
// initialise the pin directions
// ----------------------------------------------------------------------------

void mdm_init(void){ 
	bRB2 = 0;
	bRB4 = 0;
	bRB5 = 0;   
	TRISB = 0xCB;
}


